#' ---
#' title: "R_Intro_iBio2019"
#' author: "T.Moyano & J.Maldonado"
#' date: "8 de agosto de 2019"
#' output:
#'   pdf_document: default
#'   html_document: default
#' ---
#' 
#' # INTRODUCCIÓN a R
#' [iBio] <http://www.ibio.cl/2019/08/06/workshop-ii-ibio-bioinformatica>  [iBio](http://www.ibio.cl/2019/08/06/workshop-ii-ibio-bioinformatica)
#' 
#' #### Para insertar un "comentario" en el código se utiliza el símbolo gato (#).
#' #### Cuando está presente, todo lo que está a la derecha del (#) no es interpretado por R.
#' #### De esta forma podemos evitar que se ejecute un código y también podemos dejar comentarios de ayuda. 
#' 
## ------------------------------------------------------------------------
# 2+2

#' 
#' #### R puede realizar funcionas matemáticas básicas:
#' 
## ------------------------------------------------------------------------
2+2

2*3

8/2

10-3

#' 
#' #### Hay diferentes formas de asignar variables:
#' 
## ------------------------------------------------------------------------
a <- 2+2

a

b = 2+3

b

a*b

a*b -> c

#' 
#' #### Con ls() listamos los objetos o variables creadas:
#' 
## ------------------------------------------------------------------------
ls() 

#' 
#' #### Hay diferentes tipos de estructuras de datos: Vectores, Matrices, Dataframe, etc  
#' 
#' #### Creación de vectores:
#' 
## ------------------------------------------------------------------------
vectorletras<-c("plantas","hongos","milenio","iBio","cocacola")

vectorletras

vectornumeros<-c(0,1,2,3,4,5)

vectornumeros

#' 
#' #### Para ver elementos específicos de un vector:
#' 
## ------------------------------------------------------------------------
vectornumeros[3]

#' 
#' #### Operaciones matemáticas con vectores:
#' 
## ------------------------------------------------------------------------
vectornumeros2<-vectornumeros*(a+b+1)

vectornumeros2

#' 
#' #### Creación de matrices
#' 
## ------------------------------------------------------------------------
matriz1<-cbind(vectornumeros,vectornumeros2)

matriz2<-cbind(matriz1,vectorletras)

#' 
#' >Cuidado con las dimensiones
#' 
## ------------------------------------------------------------------------
vectorletras<-c(vectorletras,"queso")

matriz2<-cbind(matriz1,vectorletras)

matriz2

#' 
#' >Fíjese en las comillas, ahora todo es texto.
#' 
#' #### Creación de dataframes
#' 
## ------------------------------------------------------------------------
dataframe<-data.frame(matriz1,vectorletras)

dataframe[,1]

#' 
#' #### Funciones básicas
#' 
## ------------------------------------------------------------------------
sum(dataframe[,1])

dataframe[c(1:2,5:6),]

colSums(dataframe[c(1:2,5:6),1:2])

colMeans(dataframe[c(1:2,5:6),1:2])

#' 
#' #### Para obtener ayuda acerca de funciones se puede usar el signo de pregunta seguido del comando:
## ------------------------------------------------------------------------
?colMeans

#' 
#' ### Cuando trabajamos en R, podemos elegir el directorio de trabajo.
#' 
#' #### Configurar el directorio de trabajo (origen y destino de los archivos de la sesión)
#' 
#' * Modo gráfico (Session -> Set Working Directory -> Choose Directory)
#' 
#' * Modo comando
#' 
## ------------------------------------------------------------------------
setwd("D:/Lab/curso_iBio")

#' 
#' ### Usando datos tabulados
#' #### Para esta parte usaremos el set de datos "iris", el cual es un ejemplo clásico de R que es utilizado generalemente por los desarrolladores de packetes para mostrar la forma de usar las funciones de R.
#' >This famous (Fisher's or Anderson's) iris data set gives the measurements in centimeters of the variables sepal length and width and petal length and width, respectively, for 50 flowers from each of 3 species of iris. The species are Iris setosa, versicolor, and virginica.
#' 
#' #### Ejemplo1 https://rpubs.com/moeransm/intro-iris
#' #### Ejemplo2 https://www.kaggle.com/antoniolopez/iris-data-visualization-with-r
#' 
#' #### El siguiente comando les muestra los datos almacenados en la variable "iris" que está disponible siempre en R
#' 
## ------------------------------------------------------------------------
iris

#' 
#' #### Podemos copiar objetos
#' #### En este ejemplo vamos a pasar los datos de la variable de sistema "iris" a una nueva variable:
#' 
## ------------------------------------------------------------------------
tabla_iris<-iris

#' 
#' #### Ahora vamos a visualizar información del set de datos iris:
#' 
## ------------------------------------------------------------------------
plot(tabla_iris) # grafica lo que puede graficar

summary(tabla_iris) # resumen estadístico de las columnas

str(tabla_iris) # "representación textual" del objeto

dim(tabla_iris) # dimensiones del objeto

#' 
#' #### Vamos a escribirla variable "tabla_iris"" en un archivo de texto, separando las columnas por tabulaciones y sin comillas
#' 
## ------------------------------------------------------------------------
write.table(tabla_iris,"tabla_iris.txt",sep="\t",quote=F,col.names = NA)

#' 
#' #### Vamos a leer la tabla de texto que hemos creado, con los nombres en la primera columna y un encabezado.
#' #### Este ejemplo demuestra que pueden importar cualquier archivo de texto tabulado (ej csv) conociendo el separador de columnas y el diseño de la tabla.
#' 
## ------------------------------------------------------------------------
tabla_texto<-read.table("tabla_iris.txt",sep="\t",row.names = 1,header=T)

#' 
#' ### Tarea:
#' #### Usando Excel, importe el contenido del archivo tabla_iris.txt (a Excel) asegurándose que el separador de decimales usado por Excel sea el correcto para interpretar los datos como numéricos.
#' #### Hay que tener cuidado con los . y , de acuerdo al idioma.
#' 
#' ### Importando a R archivos de tipo Excel
#' #### Existen paquetes o conjuntos de funciones que se descargan de repositorios oficiales para tareas específicas o más completas que las ofrecidas por los packetes base de R:
#' #### Una de ellas es "readxl" la cual permite leer archivos ".xls" y ".xlsx".
#' #### El siguiente comando les permitirá instalar dicho paquete/función para que quede disponible en su equipo:
#' 
## ------------------------------------------------------------------------
#install.packages("readxl")

#' 
#' #### Para cargar en memoria el paquete (tenerlo disponible en la sesión actual) se usa la función "library"
#' 
## ------------------------------------------------------------------------
library(readxl)

#' 
#' #### Al leer el archivo Excel debe asegurarse que el nombre de la hoja (sheet) es el correcto. En este caso es "Hoja1".
#' 
## ------------------------------------------------------------------------

misdatos<-read_excel("ibio.xlsx",sheet = "Hoja1")

summary(as.data.frame(misdatos))

#' 
#' ### Visualizar los datos
#' #### Existen muchas funciones para visualizar datos cuya aplicación depende del tipo de datos y de lo que deseamos observar.
#' #### Revise el siguiente sitio para tener una idea de opciones disponibles: https://www.r-graph-gallery.com
#' 
#' #### Una forma habitual de revisar los datos es usar histogramas:
#' 
## ------------------------------------------------------------------------
hist(tabla_iris$Sepal.Width,nclass = 10)

?hist # ayuda de la función hist

hist(tabla_iris$Sepal.Width,nclass = 10, main="Histograma de iBio",col = "green")


#hist(tabla_iris$Species) # no se puede hace un historama de una columna que contiene sólo palabras.


mean(tabla_iris$Sepal.Length)

#' 
#' #### Para filtrar la tabla seleccionando objetos que cumplan con alguna "cualidad"" pueden utilizar el siguiente comando.
#' #### En este caso vamos ver el resultado sólo de los datos que pertenecen a la especie "virginica":
#' 
## ------------------------------------------------------------------------
mean(tabla_iris$Sepal.Length[tabla_iris$Species=="virginica"])

#' 
#' #### Prueba de Shapiro Wilk para evaluar normalidad (muestras pequeñas)
#' 
## ------------------------------------------------------------------------
hist(tabla_iris$Sepal.Length,nclass = 12)

shapiro.test(tabla_iris$Sepal.Length)

hist(tabla_iris$Sepal.Width,nclass = 12)

shapiro.test(tabla_iris$Sepal.Width)

#' 
#' #### Para ver correlación entre los datos (todas las comparaciones posibles)
#' 
## ------------------------------------------------------------------------
#cor(tabla_iris) # esto no funciona pues la matriz tiene una columna con texto

#' 
#' #### Cuidado cuando hay texto, vamos a eliminar la columna que tiene texto
#' 
## ------------------------------------------------------------------------
tabla_iris_sinespecies<-tabla_iris[,-5]

cor(tabla_iris_sinespecies)

#' 
#' #### Grafiquemos los datos pero seleccionando algunas columnas
#' 
## ------------------------------------------------------------------------
plot(tabla_iris$Sepal.Length,tabla_iris$Sepal.Width)

plot(tabla_iris$Sepal.Length,tabla_iris$Petal.Length)

#' 
#' #### Grafiquemos los datos usando boxplot y agrupando por especie y camviando el color a verde
#' 
## ------------------------------------------------------------------------
boxplot(tabla_iris$Sepal.Width~iris$Species,col ="green",main ="Especies de iris\n según la anchura del sépalo")

#' 
#' ### Guardar los gráficos
#' #### Para guardar los datos hay varias opciones... la más común es usar las opciones de la herramienta de RStudio que aparece en la parte superior de la ventana de los gráficos.
#' 
#' #### Si queremos guardar el gráfico como PDF usamos los siguientes comandos:
#' 
## ------------------------------------------------------------------------
pdf("grafico.pdf")

boxplot(tabla_iris$Sepal.Width~iris$Species,col ="green",main ="Especies de iris\nsegún la anchura del sépalo")

dev.off()

#' 
#' ### Fitro de datos
#' #### Por ejemplo, queremos los datos que no sean "virginica""
#' 
## ------------------------------------------------------------------------
#2especies<-tabla_iris[tabla_iris$Species!="virginica",] ## recordatorio de que los nombres de variables no deben comenzar con número.

#' 
#' #### ojooooo, nombres de variables en R no pueden partir con un número
#' 
## ------------------------------------------------------------------------
especies_2<-tabla_iris[tabla_iris$Species!="virginica",]

#' 
#' #### Haremos un "test de t"" para ver si las diferencias son significativas
#' 
## ------------------------------------------------------------------------
t.test(especies_2$Sepal.Width~especies_2$Species)

#' 
#' #### Guardamos este resultado en una variable para su posterior uso
#' 
## ------------------------------------------------------------------------
resultado_test_de_t<-t.test(especies_2$Sepal.Width~especies_2$Species)

#' 
#' #### Este resultado se guardó en un objeto tipo "lista""
#' #### Para visualizarlos las opciones o variables contenidas en la "lista" usamos "ls"
#' #### De esta forma podemos conocer las variables y seleccionar la que deseamos ver (en este caso, p.value)
#' 
## ------------------------------------------------------------------------
ls(resultado_test_de_t)

resultado_test_de_t$p.value

#' 
#' ### "Cruzar tablas 
#' #### Para el ejemplo, generaremos una tabla con "IDs" aleatorios.
#' 
## ------------------------------------------------------------------------
aleatorio<-sample(1:150,25)

#' 
#' #### Lo dejaremos como texto y usaremos estos "IDs" para hacer una "sub-selección" desde la tabla iris.
#' #### Con esto obtendremos una nueva tabla que es un "sub-conjunto" de la tabla iris:
#' 
## ------------------------------------------------------------------------
aleatorio<-as.character(aleatorio)

tabla_nueva<-tabla_iris[aleatorio,]

tabla_nueva

#' 
#' #### Vamos a mezclar ambas tablas. Esto producirá una tabla con las dimensiones de la tabla más pequeña (tabla_nueva), sumando las columnas de la tabla_iris. Quedarán duplicadas las columnas pero es un ejemplo.
#' 
## ------------------------------------------------------------------------
tablamezclada<-merge(tabla_nueva,tabla_iris,by.x=0,by.y=0)

dim(tablamezclada)

head(tablamezclada)

#' 
#' ### Usando funciones que recorren las tablas
#' #### Funciones por *fila* usando la "desviación estándar" como ejemplo
#' 
## ------------------------------------------------------------------------
apply(X = tabla_iris_sinespecies,1,sd)

#' 
#' #### Funciones por *columna* usando "desviación estándar" como ejemplo
#' 
## ------------------------------------------------------------------------
apply(X = tabla_iris_sinespecies,2,sd)

#' 
#' 
#' #### Funciones por *columna* usando el *promedio*
#' 
## ------------------------------------------------------------------------
apply(X = tabla_iris_sinespecies,2,mean)

#' 
#' #### Funciones por grupo (agrupando según la información contenida en la columna "Species")
#' 
## ------------------------------------------------------------------------
tapply(iris$Petal.Length, iris$Species, sd)

#' 
#' ### Creación de funciones "personales"
#' #### Una función recibe un valor y devuelve una respuesta según el valor que se ingresó
#' #### En este ejemplo, la función recibirá un valor al cual llamaremos "lo_que_entra" (puede tener cualquier nombre), y a ese valor le calculará el log2(1+X). El resultado será devuelto al usuario.
#' 
## ------------------------------------------------------------------------
funcion_ibio<-function(lo_que_entra)
{ salida= log2(lo_que_entra+1)
  return(salida)
}

#' 
#' #### Usando funciones "personales"
#' 
## ------------------------------------------------------------------------
funcion_ibio(7)

#' 
#' ### Análisis complejos
#' 
#' #### *Análisis de componentes Principales (PCA)*
#' #### En R la función del paquete base para calcular PCA es "prcomp". Hay más paquetes que permiten hacer PCA con mas o menos funciones.
#' #### Una vez realizado el cálculo nos gustaría graficar los resultados. Existen muchos paquetes de R que proveen formas de realizar gráficos, uno de ellos es autoplot del paquete ggfortify. 
#' 
## ------------------------------------------------------------------------
#install.packages("ggfortify") #instalación en caso que no lo tengan

library(ggfortify)

pca = prcomp(tabla_iris_sinespecies, scale. = T)

summary(pca)

autoplot(pca)

autoplot(pca, data = tabla_iris, colour = 'Species')

autoplot(pca, data = tabla_iris, colour = 'Species', shape = FALSE, label.size = 3)

autoplot(pca, data = tabla_iris, colour = 'Species',
         loadings = TRUE, loadings.colour = 'blue',
         loadings.label = TRUE, loadings.label.size = 3)

#' 
#' #### *Heatmap*
#' #### Al igual que en el caso de PCA, hay muchas formas de hacer un heatmap.
#' #### ACá algunos ejemplos: https://www.r-graph-gallery.com/heatmap.html
#' 
#' #### En este caso vamos a usar el paquete gplots
#' 
## ------------------------------------------------------------------------
#install.packages("gplots") #instalación en caso que no lo tengan

library(gplots)

#' 
#' #### Este comando es para cerrar los graficos en caso que exista una sesión "dev"" abierta
#' 
## ------------------------------------------------------------------------
dev.off()

#' 
#' #### Con este comando podrán abrir una ventana para la salida gráfica que es independiente del panel principal de RStudio
#' 
## ------------------------------------------------------------------------
#x11()

#' 
#' #### Asignamos los colores según especie para poder distinguirlas en el resultado del heatmap
#' #### Como los datos están ordenados, sabemos que el nombre de la especie cambia cada 50 filas por lo cual creamos un vector de colores que cambia cada 50 elementos.
#' 
## ------------------------------------------------------------------------
colores<-c(rep("blue",50),rep("red",50),rep("green",50))

#' 
#' #### Creamos el heatmap usando heatmap.2 que es una función de gplots
#' 
## ------------------------------------------------------------------------
heatmap.2(as.matrix(t(tabla_iris_sinespecies)),trace="none",colCol = colores,cex.lab=100)

#' 
#' ### Uso del paquete ggplot2
#' #### Este paquete es uno de los más utilizados para prodcir gráficos personalizados gracias a que es posible ir aplicando capas de información sobre el gráfico.
#' 
## ------------------------------------------------------------------------
#install.packages("ggplot2")
library(ggplot2)

p <-ggplot(iris,aes(x =Petal.Length,y =Petal.Width,colour =Species)) #construye el objeto que será la base del gráfico
print(p) 

p <-p+geom_point() # se agregan los puntos
print(p)

p<-p+geom_smooth() # se agregan los límites de confianza
print(p)

dev.off() #resetear la ventana de salida

p <-ggplot(iris,aes(x =Species,y =Petal.Width,colour=Species)) # gráfico de cajas
p <-p+ geom_boxplot()
print(p)

#' 
#' 
